@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # vim:et sts=4 sw=4
 #
 # ibus-typing-booster - A completion input method for IBus
@@ -50,23 +49,11 @@ class TabSqliteDb:
 
     “id”, “input_phrase”, “phrase”, “p_phrase”, “pp_phrase”, “user_freq”, “timestamp”
 
-    There are 2 databases, sysdb, userdb.
-
-    sysdb: “Database” with the suggestions from the hunspell dictionaries
-        user_freq = 0 always.
-
-        Actually there is no Sqlite3 database called “sysdb”, these
-        are the suggestions coming from hunspell_suggest, i.e. from
-        grepping the hunspell dictionaries and from pyhunspell.
-        (Historic note: ibus-typing-booster started as a fork of
-        ibus-table, in ibus-table “sysdb” is a Sqlite3 database
-        which is installed systemwide and readonly for the user)
-
-    user_db: Database on disk where the phrases learned from the user are stored
-        user_freq >= 1: The number of times the user has used this phrase
+    It is a database where the phrases learned from the user are stored.
+    user_freq >= 1: The number of times the user has used this phrase
     '''
     # pylint: enable=line-too-long
-    def __init__(self, user_db_file: str = '') -> None:
+    def __init__(self, user_db_file: str = 'user.db') -> None:
         global DEBUG_LEVEL # pylint: disable=global-statement
         try:
             DEBUG_LEVEL = int(str(os.getenv('IBUS_TYPING_BOOSTER_DEBUG_LEVEL')))
@@ -76,9 +63,10 @@ class TabSqliteDb:
             LOGGER.debug(
                 'TabSqliteDb.__init__(user_db_file = %s)', user_db_file)
         self.user_db_file = user_db_file
-        if not self.user_db_file and os.getenv('HOME'):
-            self.user_db_file = os.path.join(
-                str(os.getenv('HOME')), '.local/share/ibus-typing-booster/user.db')
+        if (self.user_db_file != ':memory:'
+            and os.path.basename(self.user_db_file) == self.user_db_file):
+            self.user_db_file = os.path.join(os.path.expanduser(
+                '~/.local/share/ibus-typing-booster'), self.user_db_file)
         if not self.user_db_file:
             LOGGER.debug('Falling back to ":memory:" for user.db')
             self.user_db_file = ':memory:'
@@ -382,45 +370,10 @@ class TabSqliteDb:
         if commit:
             self.database.commit()
 
-    @classmethod
-    def best_candidates(
-            cls,
-            phrase_frequencies: Dict[str, float],
-            title: bool = False) -> List[Tuple[str, float]]:
-        '''Sorts the phrase_frequencies dictionary and returns the best
-        candidates.
-
-        Should *not* change the phrase_frequencies dictionary!
-        '''
-        candidates = sorted(phrase_frequencies.items(),
-                            key=lambda x: (
-                                -1*x[1],   # user_freq descending
-                                len(x[0]), # len(phrase) ascending
-                                x[0]       # phrase alphabetical
-                            ))[:20]
-        if not title:
-            return candidates
-        candidates_title = []
-        phrases_title = set()
-        for candidate in candidates:
-            phrase = candidate[0]
-            phrase_title = phrase[:1].title() + phrase[1:]
-            if phrase_title in phrases_title:
-                continue
-            candidates_title.append((phrase_title, candidate[1]))
-            phrases_title.add(phrase_title)
-        return candidates_title
-
     def select_shortcuts(
             self,
-            input_phrase: str) -> List[Tuple[str, float]]:
-        '''
-        Get shortcuts from database completing input_phrase.
-
-        Returns a list of matches where each match is a tuple in the
-        form of (phrase, user_freq), i.e. returns something like
-        [(phrase, user_freq), ...]
-        '''
+            input_phrase: str) -> List[itb_util.PredictionCandidate]:
+        '''Get shortcuts from database completing input_phrase.'''
         input_phrase = unicodedata.normalize(
             itb_util.NORMALIZATION_FORM_INTERNAL, input_phrase)
         if DEBUG_LEVEL > 1:
@@ -443,7 +396,7 @@ class TabSqliteDb:
                  error.__class__.__name__, error)
         if results_shortcuts:
             phrase_frequencies.update(results_shortcuts)
-        best_shortcut_candidates = self.best_candidates(phrase_frequencies)
+        best_shortcut_candidates = itb_util.best_candidates(phrase_frequencies)
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'best_shortcut_candidates=%s', best_shortcut_candidates)
@@ -452,7 +405,7 @@ class TabSqliteDb:
     def select_words_empty_input(
             self,
             p_phrase: str,
-            pp_phrase: str) -> List[Tuple[str, float]]:
+            pp_phrase: str) -> List[itb_util.PredictionCandidate]:
         '''Get phrases from database which occured previously with
         any input after the given context
 
@@ -466,7 +419,7 @@ class TabSqliteDb:
                 'p_phrase=%s pp_phrase=%s', p_phrase, pp_phrase)
         phrase_frequencies: Dict[str, float] = {}
         if not p_phrase or not pp_phrase:
-            return self.best_candidates(phrase_frequencies)
+            return itb_util.best_candidates(phrase_frequencies)
         p_phrase = itb_util.remove_accents(p_phrase.lower())
         pp_phrase = itb_util.remove_accents(pp_phrase.lower())
         sqlargs = {'p_phrase': p_phrase, 'pp_phrase': pp_phrase}
@@ -483,7 +436,7 @@ class TabSqliteDb:
                 error.__class__.__name__, error)
             results = None
         if not results:
-            return self.best_candidates(phrase_frequencies)
+            return itb_util.best_candidates(phrase_frequencies)
         sqlstr = (
             'SELECT sum(user_freq) FROM user_db.phrases '
             'WHERE p_phrase = :p_phrase AND pp_phrase = :pp_phrase;')
@@ -498,21 +451,21 @@ class TabSqliteDb:
                  error.__class__.__name__, error)
             count_pp_phrase_p_phrase = 0
         if not count_pp_phrase_p_phrase:
-            return self.best_candidates(phrase_frequencies)
+            return itb_util.best_candidates(phrase_frequencies)
         for result in results:
             phrase_frequencies.update(
                 [(result[0], result[1]/float(count_pp_phrase_p_phrase))])
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'Best candidates for empty input with given context=%s',
-                self.best_candidates(phrase_frequencies))
-        return self.best_candidates(phrase_frequencies)
+                itb_util.best_candidates(phrase_frequencies))
+        return itb_util.best_candidates(phrase_frequencies)
 
     def select_words(
             self,
             input_phrase: str,
             p_phrase: str = '',
-            pp_phrase: str = '') -> List[Tuple[str, float]]:
+            pp_phrase: str = '') -> List[itb_util.PredictionCandidate]:
         '''
         Get phrases from database completing input_phrase.
 
@@ -532,7 +485,7 @@ class TabSqliteDb:
         p_phrase = itb_util.remove_accents(p_phrase.lower())
         pp_phrase = itb_util.remove_accents(pp_phrase.lower())
         title_case = input_phrase.istitle()
-        if not ' ' in input_phrase:
+        if ' ' not in input_phrase:
             # Get suggestions from hunspell dictionaries. But only
             # if input_phrase does not contain spaces. The hunspell
             # dictionaries contain only single words, not sentences.
@@ -543,7 +496,7 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'hunspell: best_candidates=%s',
-                self.best_candidates(phrase_frequencies, title=title_case))
+                itb_util.best_candidates(phrase_frequencies, title=title_case))
         # Remove the accents *after* getting the hunspell candidates.
         # If the accents were removed before getting the hunspell candidates
         # an input phrase like “Glühwürmchen” would not be added as a
@@ -614,7 +567,7 @@ class TabSqliteDb:
             # If no unigrams matched, bigrams and trigrams cannot
             # match either. We can stop here and return what we got
             # from hunspell.
-            return self.best_candidates(phrase_frequencies, title=title_case)
+            return itb_util.best_candidates(phrase_frequencies, title=title_case)
         # Now normalize the unigram frequencies with the total count
         # (which is 11 in the above example), which gives us the
         # normalized result:
@@ -636,11 +589,11 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'Unigram best_candidates=%s',
-                self.best_candidates(phrase_frequencies, title=title_case))
+                itb_util.best_candidates(phrase_frequencies, title=title_case))
         if not p_phrase:
             # If no context for bigram matching is available, return
             # what we have so far:
-            return self.best_candidates(phrase_frequencies, title=title_case)
+            return itb_util.best_candidates(phrase_frequencies, title=title_case)
         sqlstr = (
             'SELECT phrase, sum(user_freq) FROM like_input_phrase_view '
             'WHERE p_phrase = :p_phrase GROUP BY phrase;')
@@ -653,7 +606,7 @@ class TabSqliteDb:
                 error.__class__.__name__, error)
         if not results_bi:
             # If no bigram could be matched, return what we have so far:
-            return self.best_candidates(phrase_frequencies, title=title_case)
+            return itb_util.best_candidates(phrase_frequencies, title=title_case)
         # get the total count of p_phrase to normalize the bigram frequencies:
         sqlstr = (
             'SELECT sum(user_freq) FROM like_input_phrase_view '
@@ -677,11 +630,11 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'Bigram best_candidates=%s',
-                self.best_candidates(phrase_frequencies, title=title_case))
+                itb_util.best_candidates(phrase_frequencies, title=title_case))
         if not pp_phrase:
             # If no context for trigram matching is available, return
             # what we have so far:
-            return self.best_candidates(phrase_frequencies, title=title_case)
+            return itb_util.best_candidates(phrase_frequencies, title=title_case)
         sqlstr = ('SELECT phrase, sum(user_freq) FROM like_input_phrase_view '
                   'WHERE p_phrase = :p_phrase '
                   'AND pp_phrase = :pp_phrase GROUP BY phrase;')
@@ -694,7 +647,7 @@ class TabSqliteDb:
                  error.__class__.__name__, error)
         if not results_tri:
             # if no trigram could be matched, return what we have so far:
-            return self.best_candidates(phrase_frequencies, title=title_case)
+            return itb_util.best_candidates(phrase_frequencies, title=title_case)
         # get the total count of (p_phrase, pp_phrase) pairs to
         # normalize the bigram frequencies:
         sqlstr = (
@@ -721,8 +674,8 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug(
                 'Trigram best_candidates=%s',
-                self.best_candidates(phrase_frequencies, title=title_case))
-        return self.best_candidates(phrase_frequencies, title=title_case)
+                itb_util.best_candidates(phrase_frequencies, title=title_case))
+        return itb_util.best_candidates(phrase_frequencies, title=title_case)
 
     def generate_userdb_desc(self) -> None:
         '''
@@ -1275,7 +1228,6 @@ CREATE TABLE phrases (id INTEGER PRIMARY KEY, input_phrase TEXT, phrase TEXT, p_
         except Exception as error: # pylint: disable=broad-except
             LOGGER.exception('Unexpected error dumping database: %s: %s',
                               error.__class__.__name__, error)
-            return
 
     def number_of_rows_in_database(self) -> int:
         '''
