@@ -103,6 +103,12 @@ class MockEngine:
         self.mock_preedit_focus_mode = focus_mode
         self.update_preedit_text(text, cursor_pos, visible)
 
+    def hide_preedit_text(self) -> None:
+        self.update_preedit_text(IBus.Text.new_from_string(''), 0, False)
+
+    def show_preedit_text(self) -> None:
+        pass
+
     def register_properties(self, property_list: List[IBus.Property]) -> None:
         pass
 
@@ -118,6 +124,9 @@ class MockEngine:
     def connect(self, signal: str, callback_function: Any) -> None:
         pass
 
+    def add_table_by_locale(self, locale: str) -> None:
+        pass
+
 class MockLookupTable:
     def __init__(
             self,
